<?php

/**
 * Class IMS_DATABASE
 * @version 1.0.0
 */
if ( ! class_exists( 'VPP_DATABASE' ) ) {

	class VPP_DATABASE {

		/**
		 * Select
		 *
		 *      [
		 *          prefix: String
		 *          column:
		 *              [
		 *                  [
		 *                      column: String (you can set method: COUNT(*))
		 *                      as: String
		 *                  ]
		 *              ]
		 *          inner_join: class::InnerJoin()
		 *          where: class::Where()
		 *          order: class::Order()
		 *          limit: class::Limit()
		 *      ]
		 *
		 * @param string $table No prefix
		 * @param array  $arg
		 *
		 * @return mixed
		 * @version 1.0.0
		 *
		 */
		public static function Select( $table, $arg = [] ) {

			global $wpdb;

			$table = $wpdb->prefix . $table;
			$prefix = isset( $arg['prefix'] ) ? ' ' . $arg['prefix'] : null;
			$inner_join = isset( $arg['inner_join'] ) ? ' ' . $arg['inner_join'] : null;
			$where = isset( $arg['where'] ) ? ' ' . $arg['where'] : null;
			$order = isset( $arg['order'] ) ? ' ' . $arg['order'] : null;
			$limit = isset( $arg['limit'] ) ? ' ' . $arg['limit'] : null;
			$column = isset( $arg['column'] ) ? $arg['column'] : '*';

			if ( isset( $arg['column'] ) ) {

				$tmp = [];
				foreach ( $column as $index => $value ) {
					$tmp[] = $value['column'] . ( isset( $value['as'] ) ? ' ' . $value['as'] : '' );
				}
				$column = join( ',', $tmp );

			}

			$result = $wpdb->get_results( 'SELECT ' . $column . ' FROM ' . $table . $prefix . $inner_join . $where . $order . $limit );
			$out = [];

			foreach ( $result as $index => $value ) {
				$out[] = self::UnNormalize( $value );
			}

			return $out;

		}

		/**
		 * Insert
		 *
		 * @param string $table No Prefixed
		 * @param array  $arg   Convert to normalized
		 * @param null   $format
		 *
		 * @return int
		 * @version 1.0.2
		 *
		 */
		public static function Insert( $table, $arg, $format = null ) {

			global $wpdb;

			$wpdb->insert(
				$wpdb->prefix . $table,
				self::Normalize( $arg ),
				$format
			);

			return $wpdb->insert_id;

		}

		/**
		 * Delete
		 *
		 * @param string $table No prefixed
		 * @param mixed  $where class::Where()
		 *
		 * @return bool|int
		 * @version 1.0.0
		 *
		 */
		public static function Delete( $table, $where ) {

			global $wpdb;

			$sql = [ 'DELETE FROM' ];
			$sql[] = $wpdb->prefix . $table;
			$sql[] = $where;

			return $wpdb->query( join( ' ', $sql ) );

		}

		/**
		 * Update
		 *
		 * @param string $table No prefixed
		 * @param array  $value
		 * @param mixed  $where class::Where()
		 *
		 * @return bool|int
		 * @version 1.0.0
		 *
		 */
		public static function Update( $table, $value, $where ) {

			global $wpdb;

			$column = [];
			foreach ( self::Normalize( $value ) as $index => $value ) {
				$column[] = $index . '=\'' . $value . '\'';
			}

			return $wpdb->query( 'UPDATE ' . $wpdb->prefix . $table . ' SET ' . join( ',', $column ) . ' ' . $where );

		}

		/**
		 * InnerJoin
		 *
		 *      [
		 *          table: String
		 *          prefix: String
		 *          condition:
		 *              [
		 *                  left_operator: String
		 *                  right_operator: String
		 *                  operator: =  !=  >  <  >=  <= (default: =)
		 *              ]
		 *      ]
		 *
		 * @param array $arg
		 *
		 * @return string
		 * @version 1.0.0
		 *
		 */
		public static function InnerJoin( $arg = [] ) {

			global $wpdb;

			$table = $wpdb->prefix . $arg['table'];
			$condition = $arg['condition']['left_operator'] . ' ' . ( isset( $arg['condition']['operator'] ) ? $arg['condition']['left_operator'] : '=' ) . ' ' . $arg['condition']['right_operator'];

			return 'INNER JOIN ' . $table . ' ' . $arg['prefix'] . ' ON ' . $condition;

		}

		/**
		 * Create Table
		 *
		 *      [
		 *          name: String
		 *          key: Boolean
		 *          type: int(11) | varchar(150) | tinyint(4) | longtext
		 *          Collation:
		 *              [
		 *                  character_set: utf8
		 *                  collate: utf8_unicode_520_ci
		 *              ]
		 *          default: String
		 *          not_null: Boolean
		 *          auto_increment: Boolean
		 *      ]
		 *
		 * ####Version 1.1.0:
		 * #### - Add: collation
		 *
		 *
		 * @param       $table
		 * @param array $column
		 *
		 * @return bool|int
		 * @version 1.1.0
		 *
		 */
		public static function Table( $table, $column = [] ) {

			global $wpdb;

			$arg_generate = [];
			$arg_key = [];

			foreach ( $column as $key => $value ) {

				$c = '';

				if ( isset( $value['name'] ) ) {
					$c .= $value['name'];
				}
				if ( isset( $value['key'] ) ) {
					$value['key'] ? $arg_key[] = $value['name'] : null;
				}
				if ( isset( $value['type'] ) ) {
					$c .= ' ' . $value['type'];
				}
				if ( isset( $value['collation'] ) ) {

					$c .= ' CHARACTER SET ' . ( isset( $value['collation']['character_set'] ) ? $value['collation']['character_set'] : 'utf8' );
					$c .= ' COLLATE ' . ( isset( $value['collation']['utf8_unicode_520_ci'] ) ? $value['collation']['utf8_unicode_520_ci'] : 'utf8_unicode_520_ci' );

				}
				if ( isset( $value['not_null'] ) ) {
					$value['not_null'] ? $c .= ' NOT NULL' : $c .= ' NULL';
				}
				if ( isset( $value['auto_increment'] ) ) {
					$value['auto_increment'] ? $c .= ' AUTO_INCREMENT' : null;
				}
				if ( isset( $value['default'] ) ) {
					$c .= ' DEFAULT ' . $value['default'];
				}

				$arg_generate[] = $c;

			}

			$sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . $table;
			if ( ! empty( $arg_generate ) ) {
				$sql .= ' (' . join( ', ', $arg_generate ) . ( ! empty( $arg_key ) ? ', PRIMARY KEY (' . join( ',', $arg_key ) . ')' : null ) . ')';
			}

			$sql .= ' ENGINE = MYISAM';

			return $wpdb->query( $sql );

		}

		/**
		 * Where
		 *
		 *      relation: AND | OR (default: AND)
		 *      condition: (default: [])
		 *          [
		 *
		 *              // Single Condition
		 *              [
		 *                  column: String
		 *                  value: String
		 *                  operator: =  !=  >  <  >=  <= (default: =)
		 *              ],
		 *
		 *              // Group Conditions
		 *              [
		 *                  relation: AND | OR (default: AND)
		 *                  condition: (default: [])
		 *                      [
		 *                          [
		 *                              column: String
		 *                              value: String | Array
		 *                              operator: =  !=  >  <  >=  <= (default: =)
		 *                          ]
		 *                      ]
		 *              ]
		 *
		 *          ]
		 *
		 * @param array $arg
		 * @param bool  $inset
		 *
		 * @return string
		 * @version 1.2.0
		 *
		 */
		public static function Where( $arg = [], $inset = false ) {

			$relation = isset( $arg['relation'] ) ? $arg['relation'] : 'AND';
			$condition = isset( $arg['condition'] ) ? $arg['condition'] : [];
			$result = [];

			foreach ( $condition as $index => $value ) {

				if ( isset( $value['relation'] ) ) {

					$result[] = self::Where( $value, true );
					continue;

				}

				$val = ( isset( $value['value'] ) ? ( gettype( $value['value'] ) == 'array' ? '(\'' . join( '\',\'', $value['value'] ) . '\')' : '\'' . $value['value'] . '\'' ) : null );
				$result[] = $value['column'] . ' ' . ( isset( $value['operator'] ) ? $value['operator'] : '=' ) . ( isset( $value['value'] ) ? ' ' . $val : null );

			}

			$out = join( ' ' . $relation . ' ', $result );

			return $inset ? '(' . $out . ')' : 'WHERE ' . $out;

		}

		/**
		 * Order
		 *
		 *      [
		 *          column: bool (ASC: true | DESC: false)
		 *          .
		 *          .
		 *          .
		 *      ]
		 *
		 * @param $arg
		 *
		 * @return string
		 * @version 1.0.0
		 *
		 */
		public static function Order( $arg = [] ) {

			$order = [];

			foreach ( $arg as $index => $value ) {
				$order[] = $index . ' ' . ( $value ? 'ASC' : 'DESC' );
			}

			return 'ORDER BY ' . join( ',', $order );

		}

		/**
		 * Limit
		 *
		 * @param     $per_page
		 * @param int $current
		 *
		 * @return string
		 * @version 1.0.0
		 *
		 */
		public static function Limit( $per_page, $current = 1 ) {

			$start = ( $current - 1 ) * $per_page;

			return " LIMIT $start,$per_page";

		}

		/**
		 * Normalize
		 *
		 *      1. Array convert to serialized
		 *
		 *      [
		 *          index => value
		 *          .
		 *          .
		 *          .
		 *      ]
		 *
		 * @param array $arg
		 *
		 * @return array
		 * @version 1.0.0
		 *
		 */
		public static function Normalize( $arg ) {

			$result = [];

			foreach ( $arg as $index => $value ) {

				switch ( gettype( $value ) ) {

					case 'array':
						$result[ $index ] = serialize( $value );
						break;

					default:
						$result[ $index ] = $value;
						break;

				}

			}

			return $result;

		}

		/**
		 * UnNormalize
		 *
		 * @param $arg
		 *
		 * @return array
		 * @version 1.0.0
		 *
		 */
		public static function UnNormalize( $arg ) {

			$result = [];

			foreach ( $arg as $index => $value ) {

				$serialize = @unserialize( $value );
				$result[ $index ] = $serialize ? $serialize : $value;

			}

			return $result;

		}

	}

}